from aqt import mw
import os

from pathlib import Path

import uuid
import hashlib

config = mw.addonManager.getConfig(__name__)
MTCONFIG = {
        "ADDON_VERSION": "0.97",
        "ADDON_VERSION_STATS": 1000,
        "ADDON_VERSION_STATS_CONSTANT": 52349,
        "API_URL": "https://anki.meditricks.de/wp-json",
        "API_URL_MTANKI": "/mtanki/v1",
        "API_URL_TOKEN": "/jwt-auth/v1/token",
        "API_URL_RELOGIN": "/mtanki/v1/re-login",
        "API_URL_STARTTRIAL": "/mtanki/v1/start-trial",
        "TOKEN_PARAM": "?token=",
        "OS_PATH": os.path.dirname(os.path.realpath(__file__))+"/", #current_file_dir
        "WEBVIEW_PATH": f"""/_addons/{__package__}/web/""",
        "WEBVIEW_P": f"""http://127.0.0.1:{mw.mediaServer.getPort()}/_addons/""",
        "MT_RESOURCE_PATH": os.path.dirname(os.path.realpath(__file__))+"/resource/",
       # "COUNT_CARDS": config.get('COUNT_CARDS', 0),
        "SETTINGS": config.get('SETTINGS', {}),
        "is_logged_in": config.get('is_logged_in', 0),
        "deviceID": config.get('deviceID', None),

        "JWT_CODES": {
            "bad_username": "[jwt_auth] invalid_username",
            "bad_password": "[jwt_auth] invalid_password",
            #'blacklisted': "[jwt_auth] username_blacklisted"
        },
        "MT_URL_LANDING": "https://www.meditricks.de/",
        "MT_URL_PASSFORGOT": "https://www.meditricks.de/u/login?sendpass",
        "MT_URL_CREATEACC": "https://www.meditricks.de/u/signup",
        "MT_URL_SIGNUP":  "https://www.meditricks.de/u/signup",
        "MT_URL_HELP_DECKS":  "https://www.meditricks.de/anki/",
        "MT_URL_HELP_HOWTO":  "https://www.meditricks.de/anki/",


        'MT_ANKIBOX_TRIAL_FINISHED': 'finished',
        'MT_ANKIBOX_TRIAL_ONGOING': 'ongoing',

        'MT_CARD_NOTETYPE_NAMES': {
               "Zankiphil - Cloze (Zankiphil - Klinik / Projekt_Anki_Germany)": "ankizin-notetype_pre_12-2023",
               "ProjektAnkiCloze (Ankizin / Projekt_Anki_Germany)": "ankizin-projektankicloze",
               "Ankiphil - Cloze": "ankiphil-cloze"
        },


        "config": config
    }

MAINWINDOWTYPES = {
    "login": "MAINWINDOW_TYPE_LOGIN",
    "profile": "MAINWINDOW_TYPE_PROFILE",
}

def init_config():

    #print("mt: doing config-init now")
    
    config = mw.addonManager.getConfig(__name__)

    import uuid
    import base64

            
    config['COUNT_CARDS'] = 0 
    config['SETTINGS'] = {
        "cards_mtbutton_autounfold": True,
        "cards_mtbutton_autoankiboxpin": False,

        "cards_dev_emulate_iframe": False,
        "cards_dev_emulate_iframe_gdpr": False
        }
    config['consent_agb'] = 0 
    config['did_init'] = 1
    config['is_logged_in'] = 0


    MTCONFIG['SETTINGS'] = config['SETTINGS']

    MTCONFIG['config'] = config
    mw.addonManager.writeConfig(__name__, config)

    generateNewDeviceIDsAnd(True)



def copyMediaToAppMedia():
    for file in Path(MTCONFIG.get("MT_RESOURCE_PATH")).iterdir():
        if not mw.col.media.have(file.name):
            mw.col.media.add_file(str(file.absolute()))
        mw.col.media.add_file(str(file.absolute()))


def generateNewDeviceIDsAnd(save = False):
    deviceID = generateDeviceID()
    #print("got new deviceID", deviceID)
    deviceID2 = base64encode(deviceID)
    if save is True:
        saveDeviceIDs(deviceID,deviceID2)
    return [deviceID,deviceID2]
        
def saveDeviceIDs(deviceID, deviceID2):
    #print("saving deviceIDs")
    #print(deviceID)
    #print(deviceID2)
    config = mw.addonManager.getConfig(__name__)
    config['deviceID'] = deviceID
    config['deviceID2'] = deviceID2

    MTCONFIG['deviceID'] = config['deviceID']
    MTCONFIG['deviceID2'] = config['deviceID2']

    mw.addonManager.writeConfig(__name__, config)
        
def generateDeviceID(base64 = False):
        
        from datetime import datetime


        # Unique machine identifier
        machine_id = uuid.getnode()

        current_datetime = datetime.now().strftime("%Y%m%d%H%M%S")  # Format: YYYYMMDDHHMMSS

        machine_id = f"{machine_id}_{current_datetime}"
        machine_id = hashlib.sha256(machine_id.encode()).hexdigest()




         # Generate a unique identifier based on user's Anki config folder path
        # This assumes that the path contains elements unique to each installation/user
        config_path = os.path.abspath(os.curdir)
        config_id = hashlib.sha256(config_path.encode()).hexdigest()
        

        # Combine the two to create a unique device ID
        device_id = f"{machine_id}-{config_id}"
        

        if base64:

            device_id = base64encode(device_id)

        return device_id

def base64encode(string):
     
            import base64
            # Encode the string into bytes
            byte_string = string.encode('utf-8')

            # Base64 encode the byte string
            base64_bytes = base64.b64encode(byte_string)

            # Decode the Base64 bytes to a string
            base64_string = base64_bytes.decode('utf-8').rstrip('=')

            return base64_string


def base64decode(base64_string):
     
            import base64
            # Add back the padding '=' characters
            padding = 4 - (len(base64_string) % 4)
            base64_string_padded = base64_string + ('=' * padding)

            # Decode the Base64 string back to bytes
            decoded_bytes = base64.b64decode(base64_string_padded)

            # Decode the bytes back to the original string
            original_string = decoded_bytes.decode('utf-8')
            
            return original_string