

from aqt import mw, QObject, pyqtSlot, QThreadPool


from .config import MTCONFIG, generateNewDeviceIDsAnd, saveDeviceIDs


from .mt_window import MTWindow

from.mt_token import MTTokenHandler

from .login_tasks import LoginTask, ReLoginTask, StartTrialTask


import json

config = MTCONFIG
        

#class MyWebChannel(QWebChannel):
    


class MTLoginHandler(QObject):
    def __init__(self, appContext): 
        super().__init__()
        self.MTTokenHandler = MTTokenHandler()
        self.appContext = appContext

    
    @pyqtSlot(str, str)
    def doLogin(self, username, password):
        
        #generate new on every login
        deviceIDs = generateNewDeviceIDsAnd()
        MTCONFIG['temp_ID'] = deviceIDs
        url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_TOKEN"]
        self.login_task = LoginTask(url, username, password, deviceIDs[0])


        self.login_task.signals.result.connect(self.onLoginCompleted)
        QThreadPool.globalInstance().start(self.login_task)

    
    @pyqtSlot(str, str)
    def doReLogin(self, token):
        
        #generate new on every login
        deviceIDs = generateNewDeviceIDsAnd()
        MTCONFIG['temp_ID'] = deviceIDs
        url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_RELOGIN"]
        self.re_login_task = ReLoginTask(url, token)


        self.re_login_task.signals.result.connect(self.onReLoginCompleted)
        QThreadPool.globalInstance().start(self.re_login_task)

    @pyqtSlot()
    def doStartTrial(self):
        
        url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_STARTTRIAL"]

        token = self.MTTokenHandler.get_token_string_full()


        self.starttrial_task = StartTrialTask(url, token)


        self.starttrial_task.signals.result.connect(self.onTrialStarted)
        QThreadPool.globalInstance().start(self.starttrial_task)


    
    @pyqtSlot(bool, str, str)
    def onLoginCompleted(self, success, errorcode = "", message = ""):
        ##print("inside onlogincompleted")
        if success:
            # Update UI for successful login
            ##print("got signal success")
            token = json.loads(errorcode)
            self.MTTokenHandler.save_token(token)
            self.appContext.login_display_result_success(errorcode)
            #write to config (for autounfold to start)
            config = mw.addonManager.getConfig(__name__)
            MTCONFIG['is_logged_in'] = 1
            config['is_logged_in'] = 1
            mw.addonManager.writeConfig(__name__, config)
            saveDeviceIDs(token['ID'],token['ID2'])
        else:
            # Update UI for failed login
            #self.appContext.loginSuccess(False)
            #print("got signal fail")
            #display error message
            self.appContext.login_display_result_fail(errorcode, message)

        ##print("errorcode: ", errorcode)
        ##print("message: ", message)


    
    @pyqtSlot(bool, str, str)
    def onReLoginCompleted(self, success, token = "", message = ""):
        ##print("inside onRElogincompleted")
        if success:
            # Update UI for successful login
            
            
            token = json.loads(token)
            
            
            self.MTTokenHandler.save_token(token)
            
            
            self.appContext.profile_display_relogin_success(message)
        else:
            # Update UI for failed login
            
            #print("got signal fail")
            #print(success)
            #print(token)
            #print(message)
            #display error message
            self.appContext.profile_display_relogin_fail(message)

        ##print("errorcode: ", token)
        ##print("message: ", message)

    
    @pyqtSlot(bool, str, str)
    def onTrialStarted(self, success, trial_check, trial_started):
        ##print("inside ontrialstarted")
        if success:
            if trial_started == "true":
                config = mw.addonManager.getConfig(__name__)
                config['user']['trial-status'] = MTCONFIG['MT_ANKIBOX_TRIAL_ONGOING']
                mw.addonManager.writeConfig(__name__, config)
                mw.MTAppContext.update_startTrialSuccess()
            # Update UI for successful login
                
            ##print("got signal success")
       # else:
            # Update UI for failed login
            
            ##print("got signal fail")
            #display error message

            
    def check_token_and_load_login(self, _=None):
        if not self.tokenHandler.is_token_present():
            ##print("Token not present. Loading login window...")
            mw.mtmain = MTWindow()
            mw.mt_login_window.load_login_page()
            mw.mt_login_window.show()
            return

        config = mw.addonManager.getConfig(__name__)
        token = config.get('t')
        
        ##print("LoginHandler: Token is present.")
        mw.mt_profile_window = MTWindow()
        mw.mt_profile_window.load_profile_page()
        mw.mt_profile_window.show()


#obsolete
    def validate_token_and_load_login(self, _=None):
        if not self.tokenHandler.is_token_present():
            ##print("Token not present. Loading login window...")
            mw.mt_login_window = MTWindow()
            mw.mt_login_window.show()
            return

        config = mw.addonManager.getConfig(__name__)
        token = config.get('t')

        if not self.tokenHandler.validate_token(token):
            ##print("Token is invalid or expired. Loading login window...")
            mw.mt_login_window = MTWindow()
            mw.mt_login_window.load_login_page()
            mw.mt_login_window.show()
            return
        
        ##print("LoginHandler: Token is valid.")
        mw.mt_profile_window = MTWindow()
        mw.mt_profile_window.load_profile_page()
        mw.mt_profile_window.show()
        

        