
import requests

from aqt import QObject, pyqtSignal, pyqtSlot, QRunnable

from .config import MTCONFIG

import json

config = MTCONFIG



# Function to search for the string in the dictionary values
def search_in_dict(dictionary, target):
    for key, value in dictionary.items():
        if value == target:
            return key
    return False


class LoginSignals(QObject):
    result = pyqtSignal(bool, str, str)

class ReLoginSignals(QObject):
    result = pyqtSignal(bool, str, str)


class StartTrialSignals(QObject):
    result = pyqtSignal(bool, str, str)

class LoginTask(QRunnable):
    #login_completed = pyqtSignal(bool)

    def __init__(self, url, username, password, deviceID):
        super().__init__()
        self.url = url
        self.username = username
        self.password = password
        self.deviceID = deviceID
        self.signals = LoginSignals()


    @pyqtSlot()
    def run(self):
        # URL for your WordPress REST API endpoint
        #url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_TOKEN"]
        #print(f"url {self.url}")
        try:
            
            response = requests.post(self.url, json={"username": self.username, "password": self.password, "ID": self.deviceID})

            message = ""
            errorcode = ""
            
            if response.status_code == 200:
                token = json.dumps(response.json().get('token'))
                data = response.json().get('data')
                ##print("Login successful. Token:", token)
                # Here, you should save the token for later use
                #self.MTTokenHandler.save_token(token)
                #self.login_completed.emit(True)
                self.signals.result.emit(True, token, message )
            else:
                ##print("Login failed:", response.text)
                json_response = json.loads(response.text)
                ##print("Login json failed:", json_response)
                ##print("code:", json_response.get('code'))
                jwt_errorcode =  search_in_dict(MTCONFIG.get('JWT_CODES'),  json_response.get('code'))
                ##print("translated into ",jwt_errorcode )
                if jwt_errorcode is not False :
                    errorcode = jwt_errorcode
                else:
                    message = json_response.get('message')
                ##print("translated into ",errorcode )
                #self.login_completed.emit(False)

                self.signals.result.emit(False, errorcode, message )

        except requests.exceptions.RequestException as e:
            print("Error during login:", e)


class StartTrialTask(QRunnable):
    #login_completed = pyqtSignal(bool)

    def __init__(self, url, token):
        super().__init__()
        self.url = url
        self.token = token
        self.signals = StartTrialSignals()

        ##print("init logintask")
        ##print(url)

    @pyqtSlot()
    def run(self):
        # URL for your WordPress REST API endpoint
        #url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_TOKEN"]
        ##print(f"url {self.url}")
        try:
            
            response = requests.post(self.url, json={"token":self.token})

            message = ""
            errorcode = ""
            
            if response.status_code == 200:
                trial_check = json.dumps(response.json().get('trial_check'))
                trial_started = json.dumps(response.json().get('trial_started'))
                data = response.json().get('data')
                ##print("starttrial result successful.")
                ##print("trial_check", trial_check)
                ##print("trial_started", trial_started)
                
                self.signals.result.emit(True, trial_check, trial_started )
            else:
                ##print("Login failed")
                ##print("Login failed:", response.text)
                json_response = json.loads(response.text)
                ##print("Login json failed:", json_response)
                ##print("code:", json_response.get('code'))
                jwt_errorcode =  search_in_dict(MTCONFIG.get('JWT_CODES'),  json_response.get('code'))
                ##print("translated into ",jwt_errorcode )
                if jwt_errorcode is not False :
                    errorcode = jwt_errorcode
                else:
                    message = json_response.get('message')
                ##print("translated into ",errorcode )
                    

                self.signals.result.emit(False, errorcode, message )

        except requests.exceptions.RequestException as e:
            print("Error during login:", e)

class ReLoginTask(QRunnable):
    #login_completed = pyqtSignal(bool)

    def __init__(self, url, token):
        super().__init__()
        self.url = url
        self.token = token
        self.signals = ReLoginSignals()

        ##print("init relogintask")

    @pyqtSlot()
    def run(self):
        # URL for your WordPress REST API endpoint
        #url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_TOKEN"]
        ##print(f"url {self.url}")
        ##print(f"token {self.token}")
        try:
            
            response = requests.post(self.url, json={"token": self.token})

            message = ""
            errorcode = ""
            
            if response.status_code == 200:
                
                data = response.json().get('data')
                if data == "error":
                    ##print("relogin failed")
                    error_msg = response.json().get('error_msg')
                    ##print(error_msg)
                    self.signals.result.emit(False, "", error_msg )
                elif data == "success":
                    #SUCCESS
                    token = json.dumps(response.json().get('new_token').get("token"))
                    ##print("Login successful. Token:", token)
                    # Here, you should save the token for later use
                    #self.MTTokenHandler.save_token(token)
                    #self.login_completed.emit(True)
                    self.signals.result.emit(True, token, message )
            else:
                
                ##print("Login failed:", response.text)
                json_response = json.loads(response.text)
                ##print("Login json failed:", json_response)
                ##print("code:", json_response.get('code'))
                jwt_errorcode =  search_in_dict(MTCONFIG.get('JWT_CODES'),  json_response.get('code'))
                ##print("translated into ",jwt_errorcode )
                if jwt_errorcode is not False :
                    errorcode = jwt_errorcode
                else:
                    message = json_response.get('message')
                ##print("translated into ",errorcode )
                    


        except requests.exceptions.RequestException as e:
            print("Error during login:", e)