

from aqt import mw, gui_hooks


from .config import MTCONFIG

import csv, os

notetypes = [
    "ProjektAnkiCloze (Ankizin / Projekt_Anki_Germany)",
    "Zankiphil - Cloze (Zankiphil - Klinik / Projekt_Anki_Germany)"
]

def substitute_variables(file_path, variables):
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            content = file.read()

        for key, value in variables.items():
            content = content.replace(f'[[[{key}]]]', value)
    except:
        content = ""

    return content

class MTCardHandler:

    def __init__(self, appContext, tokenHandler):
        self.appContext = appContext
        self.MTTokenHandler = tokenHandler
        #print("adding prepareMThtml to gui_hooks")
        gui_hooks.card_will_show.append(self.prepareMTHTML)


    def checkForSupportedNoteType(self, card):
        card_return_notetype_name = None
        card_note_type_name = card.note_type().get("name")
        mt_card_notetype_names = MTCONFIG.get("MT_CARD_NOTETYPE_NAMES")

        for notetype_key, notetype_value in mt_card_notetype_names.items():
            if card_note_type_name.startswith(notetype_key):
                card_return_notetype_name = notetype_value
                break
        return card_return_notetype_name

    def prepareMTHTML(self, html, card, context):
            card_notetype_supported = True
            
            
            card_notetype_name = self.checkForSupportedNoteType(card)
            if card_notetype_name is None:
                card_notetype_supported = False
            

          #  #print("card_notetype_name")
          #  #print(card_notetype_name)

            #only do in answer views
            if context not in ["reviewAnswer"]:#, "previewAnswer"]:
                return html
            
            token = self.MTTokenHandler.get_token_string()
            
            ankiboxid = self.MTTokenHandler.get_deviceID()


            config = mw.addonManager.getConfig(__name__)
    
            autounfold = config.get('SETTINGS').get("cards_mtbutton_autounfold")

            autoankiboxpin = config.get('SETTINGS').get("cards_mtbutton_autoankiboxpin")

            emulate = config.get('SETTINGS').get("cards_dev_emulate_iframe")
            emulate_gdpr = config.get('SETTINGS').get("cards_dev_emulate_iframe_gdpr")


            # STYLES - to be included in deck


            # Construct the path
            current_file_dir = os.path.dirname(os.path.realpath(__file__))
            scripts_path = os.path.join(current_file_dir, "card-templates", "styles.html")

            
            variables = {
            }
            styles_html = substitute_variables(scripts_path, variables)

            html = html+styles_html


            # SCRIPTS - to be included in deck
            iframeresizerPath = MTCONFIG.get('WEBVIEW_PATH')+"/js/iframeResizer.min.js"


            # Construct the path
            current_file_dir = os.path.dirname(os.path.realpath(__file__))
            scripts_path = os.path.join(current_file_dir, "card-templates", "scripts.html")

            
            variables = {
                'iframeresizerPath': iframeresizerPath,
            }
            scripts_html = substitute_variables(scripts_path, variables)
            

            html = html+scripts_html

            ##emulate 
            # check if existing ID-db
            hasMTContent = False
            if self.check_noteID(str(card.nid)):
                hasMTContent = True
                #print("MT SHOULD HAVE MTCONTENT")


            emulation_html = ""
            if card_notetype_supported:
                # Construct the path
                current_file_dir = os.path.dirname(os.path.realpath(__file__))
                file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_emulate.html""")
                variables = {
                    'card_nid': str(card.nid),
                }
                emulation_html = substitute_variables(file_path, variables)


            if emulate and hasMTContent:
                
                #print("MT SHOULD EMULATE")
                html = html+emulation_html

                if card.note_type().get("name").startswith("Ankiphil"):
                    if emulate_gdpr:
                        ##print("EMULATING GDPR")
                        
                        current_file_dir = os.path.dirname(os.path.realpath(__file__))
                        file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_gdpr.html""")
                        variables = {
                            #'var2': 'value2'
                        }
                        gdpr_content_html = ""#substitute_variables(file_path, variables)

                        current_file_dir = os.path.dirname(os.path.realpath(__file__))
                        file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_gdpr_styles.html""")
                        variables = {
                            #'var2': 'value2'
                        }
                        gdpr_styles_html = substitute_variables(file_path, variables)

                        current_file_dir = os.path.dirname(os.path.realpath(__file__))
                        file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_gdpr_scripts.html""")
                        variables = {
                            #'var2': 'value2'
                        }
                        gdpr_scripts_html = substitute_variables(file_path, variables)


                        # Construct the path
                        current_file_dir = os.path.dirname(os.path.realpath(__file__))
                        file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_gdpr_emulate.html""")
                        variables = {
                            'gdpr_content': gdpr_content_html.replace("\n", "").replace("\r", "")
                            #'var2': 'value2'
                        }
                        gdpr_emulation_html = substitute_variables(file_path, variables)
                        html = html + gdpr_emulation_html + gdpr_styles_html + gdpr_scripts_html
                    else:
                        ##print("EMULATING GDPR-no")

                        html = html +  """
                                        <script>window.MT_gdpr_approved = true;</script>"""

                
            # tech test, injectionobserver

            
            emulate_observer = ""
            if card_notetype_supported:
                # Construct the path
                current_file_dir = os.path.dirname(os.path.realpath(__file__))
                file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_test-observer.html""")
                variables = {
                }
                emulate_observer = substitute_variables(file_path, variables)
                
                if card.note_type().get("name").startswith("Ankiphil"):
                
                        current_file_dir = os.path.dirname(os.path.realpath(__file__))
                        file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_gdpr_scripts.html""")
                        variables = {
                            #'var2': 'value2'
                        }
                        gdpr_scripts_html = substitute_variables(file_path, variables)
                        html = """
                                        <script>window.MT_gdpr_approved = true;</script>""" + html + gdpr_scripts_html
                


            html = html + emulate_observer

            if token is False:
                
                token_html = """
                                url.searchParams.set('token', 'true');
                                """
            else:
                token_html = """
                                url.searchParams.set('token', '"""+token+"""."""+ankiboxid+"""');
                                """
            addonversion = MTCONFIG.get('ADDON_VERSION_STATS')
            addonversion_html = """
                                url.searchParams.set('aversion', '"""+str(addonversion)+"""');
                                    """
            token_html = token_html + addonversion_html
            dounfold = False
            autofoldclick = ""

            if autounfold:
               # if MTCONFIG.get('is_logged_in'):
                    dounfold = True
                    
            if dounfold is True:

                autofoldclick = ""
                if card_notetype_supported:
                    # Construct the path
                    current_file_dir = os.path.dirname(os.path.realpath(__file__))
                    file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_autofoldclick.html""")
                    variables = {
                    }
                    autofoldclick = substitute_variables(file_path, variables)
                html = html + autofoldclick
            #else:
                ##print("NO do auto unfold")


            
            # Construct the path
            current_file_dir = os.path.dirname(os.path.realpath(__file__))
            file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}.html""")
            variables = { 
                         'card_ord':str(card.ord),
                         'token_html':token_html
            }
            html_mt = substitute_variables(file_path, variables)

            html = html + html_mt
            
            if autoankiboxpin is True:
                
                # Construct the path
                current_file_dir = os.path.dirname(os.path.realpath(__file__))
                file_path = os.path.join(current_file_dir, "card-templates", f"""{card_notetype_name}_autoankiboxpin.html""")
                variables = {
                }
                html_autoboxpin = substitute_variables(file_path, variables)
                html = html + html_autoboxpin

            return html + html_mt



    def check_noteID(self, note_id):
        relative_path = 'data/_meditricks_mtsp_nf_v1000_complete.csv'
        try:
            full_path = os.path.join(os.path.dirname(__file__), relative_path)
            
            with open(full_path, mode='r', encoding='utf-8') as file:
                csv_reader = csv.reader(file)
                for row in csv_reader:
                    if note_id in row:
                        return True
            return False
        except Exception:
            print()

        return False