



from aqt import mw, QObject

from datetime import datetime

from .config import MTCONFIG

from .mt_main_router import MTMainRouter
from .toolbar import MTToolbarHandler
from .mt_window import MTWindow
from .mt_token import MTTokenHandler
from .login import MTLoginHandler
from .mt_card_handler import MTCardHandler


class MTMainContext(QObject):

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(MTMainContext, cls).__new__(cls, *args, **kwargs)
        return cls._instance
    
    def __init__(self):
        
        self.MTMainRouter = MTMainRouter(self)
        self.MTToolbarHandler = MTToolbarHandler()
        self.MTTokenHandler = MTTokenHandler()
        self.MTLoginHandler = MTLoginHandler(self)
        self.MTCardHandler = MTCardHandler(self, self.MTTokenHandler)
        self.config = MTCONFIG
        self.MTMainWindow = MTWindow()
        mw.MTAppContext = self

    def bootProcedure(self):
        #start all the listeners
    
        from aqt.gui_hooks import (
            top_toolbar_did_init_links,
            webview_did_receive_js_message,
            webview_will_set_content,
            top_toolbar_will_set_left_tray_content)
        

        #any pycmd commands
        webview_did_receive_js_message.append(self.MTMainRouter._on_pycmd_message)

        #toolbar html
        top_toolbar_will_set_left_tray_content.append(self.MTToolbarHandler.on_top_toolbar_will_set_tray_content)
        #toolbar css
        webview_will_set_content.append(self.MTToolbarHandler.on_webview_will_set_content)
        

    def getAccessText(self, access):
        #print(access)
        #print("inside get access text")
        expire = access[0].get("expire_date")
        
        date_obj = datetime.strptime(expire, "%Y-%m-%d")

        expire = date_obj.strftime("%d.%m.%Y")
        text = f"""Dein Lernpaket ist bis <b>{expire}</b> g&uuml;ltig."""
        return text
    
    #handling LOGIN

    def doLogin(self, username, password):
        self.MTLoginHandler.doLogin(username, password)

    def doReLogin(self):

        token = self.MTTokenHandler.get_token_string_full()
        self.MTLoginHandler.doReLogin(token)

    
    #handling LOGIN

    def doStartTrial(self):
        self.MTLoginHandler.doStartTrial()


    #callbacks from LOGIN


    def login_display_result_success(self, name):
        #print("in appcontext now success")
        #TODO check if window is still displayed
        self.MTMainWindow.login_display_result_success("NAME")


    def login_display_result_fail(self, errorcode, message):
        #print("in appcontext now fail")
        #TODO check if window is still displayed
        self.MTMainWindow.login_display_result_fail( errorcode, message)

        #callbacks from PROFILE

    def profile_display_relogin_success(self, name):
        #print("in appcontext now success")
        #TODO check if window is still displayed
        self.MTMainWindow.profile_display_relogin_success("NAME")


    def profile_display_relogin_fail(self, message):
        #print("in appcontext now fail")
        #TODO check if window is still displayed
        self.MTMainWindow.profile_display_relogin_fail(message)


    def login_ui_finished(self):
        #print("in appcontext now")
        #TODO check if window is still displayed
        self.MTMainWindow.load_profile_page()

    def update_startTrialSuccess(self):
        self.MTMainWindow.update_startTrialSuccess()

    #callbacks from LOGIN end
    
    #callbacks from pycmd
    def clickedToolbar(self):
        #open profile window
        self.check_token_and_load_loginprofile()

    def destroyLogin(self):
        self.doLogout()

    
    def doLogout(self):
        self.MTTokenHandler.delete_token()
        

    
    def open_login_window(self):
        mw.mt_loginHandler = LoginHandler()
        mw.mt_loginHandler.check_token_and_load_login()


    def check_token_and_load_loginprofile(self, _=None):
        if not self.MTTokenHandler.is_token_present():
            #print("Token not present. Loading login window...")
            self.MTMainWindow.load_login_page()
            self.MTMainWindow.show()
            return

        
        #print("LoginHandler: Token is present.")
        self.MTMainWindow.load_profile_page()
        self.MTMainWindow.show()

    def writeSettings(self, key, state):
        config = mw.addonManager.getConfig(__name__)
        config['SETTINGS'][key] = state
        mw.addonManager.writeConfig(__name__, config)


    #upvote counter
    def upvote_MTCounter(self):
        config = mw.addonManager.getConfig(__name__)
        config["user"]['view-count'] = config["user"]['view-count'] + 1
        #print("upvoting internally +1")
        # Save the updated config
        mw.addonManager.writeConfig(__name__, config)