
from aqt import mw
import json

from .config import MTCONFIG, base64decode, base64encode

class MTTokenHandler:
    def is_token_present( _=None):
        config = mw.addonManager.getConfig(__name__)
        token = config.get('t', None)
        return bool(token)
    
    def get_token( _=None):
        config = mw.addonManager.getConfig(__name__)
        token = config.get('t', None)

        try:

            token = base64decode(token)
            token = json.loads(token)

            #decode b64
            data = token['data']
            data['user_firstname'] = base64decode(data['user_firstname'])
            data['user_email'] = base64decode(data['user_email'])
            token['data'] = data
            
        except Exception as e:
            print("error decoding token")

        return token
    
    def get_token_string(self):
        token = self.get_token()
        #print("get_token_tring")
        #print(token)
        if isinstance(token, dict):
            return token['token']
        return False
    
    def get_token_string_full(self):
        token = self.get_token()
        deviceID = self.get_deviceID()
        #print("get_token_tring")
        #print(token)
        if isinstance(token, dict):
            return token['token']+"."+deviceID
        return False
    
    def get_deviceID(self):
        token = self.get_token()
        if isinstance(token, dict):
            return token['deviceID2']
        return False

    def save_token(self, token):
        #print("saving token in tokenhandler")
        
        config = mw.addonManager.getConfig(__name__)

        #decode b64
        data = token['data']
        try:
            data['user_firstname'] = base64encode(data['user_firstname'])
            data['user_email'] = base64encode(data['user_email'])
            #data['trialstatus'] = base64encode(data['trial-status'])
            #data['viewcount'] = base64encode(data['view-count'])
            token['data'] = data
            token['deviceID'] = token['ID']
            token['deviceID2'] = token['ID2']

            
            json_string = json.dumps(token)
            token = base64encode(json_string)

        except Exception as e:
            print("error encoding token")
            #print(e)

        config['user'] = {}
        config['user']['trial-status'] = data['trial-status']
        config['user']['view-count'] = int(data['view-count'])
        config['t'] = token
        mw.addonManager.writeConfig(__name__, config)
    
    def delete_token(self):
        #print("MT: delete token")
        config = mw.addonManager.getConfig(__name__)
        try:
            config.pop('t', None)  # Use pop instead of del for safer removal
            config.pop('user', None)
        except Exception as e:
            print(f"An error occurred: {e}")

        #write to config (for autofunold)
        MTCONFIG['is_logged_in'] = 0
        config['is_logged_in'] = 0
        mw.addonManager.writeConfig(__name__, config)



    #obsolete?    
    def validate_token(self, token):
        url = MTCONFIG["API_URL"] + MTCONFIG["API_URL_MTANKI"] + "/validate"
        data = {"token": token['token']}
        try:
            response = requests.post(url, json=data)
            response.raise_for_status()  # Raises an HTTPError if the HTTP request returned an unsuccessful status code
            if response.status_code == 200:
                response_json = response.json()  # Parse the JSON response

                # Check if 'token' is in the response and 'valid' key is True
                if 'token' in response_json and response_json['token'].get('valid') == True:
                    #print("TokenValidator: Token is valid.")
                    # Do something here if the token is valid
                    return True
                else:
                    #print("Token is not valid or not present in the response.")
                    # Handle the case where the token is invalid or not present
                    return False
        except requests.RequestException as e:
            print(f"Error during token validation: {e}")
            return False