

from aqt import mw, QMainWindow, QWebEngineView, QWebEnginePage, QWebChannel, QUrl, QDesktopServices
import os

from datetime import datetime

from .mt_token import MTTokenHandler
from .mt_window_backend import MTWindowBackend

from.config import MTCONFIG, MAINWINDOWTYPES


class MTWebEnginePage(QWebEnginePage):
    def acceptNavigationRequest(self, url, nav_type, is_main_frame):
        # If it's a link click, open in external browser
        if nav_type == QWebEnginePage.NavigationType.NavigationTypeLinkClicked:
            QDesktopServices.openUrl(url)
            return False
        return super(MTWebEnginePage, self).acceptNavigationRequest(url, nav_type, is_main_frame)



class MTWindow(QMainWindow):

    
    def __init__(self):
        super().__init__()
        self.resize(800, 600)  # Set a default window size for testing
        self.currentWindowType = ""
        self.initBackend()


    def initBackend(self):
        try:
                #print("Initializing MTWindowBackend...")
                
                self.browser = QWebEngineView()
                self.setCentralWidget(self.browser)

                self.browser.setPage(MTWebEnginePage(self.browser))

                self.channel = QWebChannel()
                self.backend = MTWindowBackend(self)
                self.channel.registerObject('py', self.backend)
                self.browser.page().setWebChannel(self.channel)
                
        except Exception as e:
            print("Error in init MTWindow:", e)

    def centerWindow(self):
        try:
            main_window_geometry = mw.frameGeometry()

            # Calculate the center point
            centerPoint = main_window_geometry.center()

            # Adjust frame geometry of MTWindow
            frame_geometry = self.frameGeometry()
            frame_geometry.moveCenter(centerPoint)

            self.move(frame_geometry.topLeft())
        except:
            return
         

    def load_login_page(self):

        if self.currentWindowType == MAINWINDOWTYPES.get('login'):
            # Bring the window to the foreground
            self.raise_()
            self.activateWindow()

            self.centerWindow()
            return
        try:
                current_file_dir = os.path.dirname(os.path.realpath(__file__))

                login_html_path = os.path.join(current_file_dir, "web", "login.html")
                base_url = QUrl.fromLocalFile(os.path.join(current_file_dir, 'web') + '/')

                with open(login_html_path, 'r', encoding='utf-8') as f:
                
                    html = f.read()

                url_passw = MTCONFIG.get('MT_URL_PASSFORGOT')
                url_createacc = MTCONFIG.get('MT_URL_HELP_HOWTO') + "?aversion="+str(MTCONFIG.get('ADDON_VERSION_STATS') * MTCONFIG.get('ADDON_VERSION_STATS_CONSTANT'))
                url_landing = MTCONFIG.get('MT_URL_LANDING') + "?aversion="+str(MTCONFIG.get('ADDON_VERSION_STATS') * MTCONFIG.get('ADDON_VERSION_STATS_CONSTANT'))


                addon_version = MTCONFIG.get("ADDON_VERSION")


                # Inject the data into the HTML
                html = html.format(addon_version=addon_version, MT_URL_PASSFORGOT=url_passw, MT_URL_LANDING=url_landing, MT_URL_CREATEACC=url_createacc,  MT_URL_SIGNUP=url_createacc,CP_YEAR=datetime.now().year)

                self.browser.setHtml(html, base_url)
                #print("MTWindow initialized successfully.")
                self.currentWindowType = MAINWINDOWTYPES.get('login')

                self.raise_()
                self.activateWindow()

        except Exception as e:
            print("Error in loading MTWindow:", e)


        


    def load_profile_page(self):


        self.resize(1000, 850)
        self.centerWindow()

        if self.currentWindowType == MAINWINDOWTYPES.get('profile'):
             # Bring the window to the foreground
            self.raise_()
            self.activateWindow()
            self.profile_updateViewCounter()

            self.centerWindow()
            return
        
        current_file_dir = os.path.dirname(os.path.realpath(__file__))

        profile_html_path = os.path.join(current_file_dir, "web", "profile.html")

        # Check if the HTML file exists
        if not os.path.exists(profile_html_path):
            print("Error: 'profile.html' does not exist.")
            # Handle the error appropriately

        base_url = QUrl.fromLocalFile(os.path.join(current_file_dir, 'web') + '/')
        with open(profile_html_path, 'r', encoding='utf-8') as f:
            html = f.read()

        if MTTokenHandler().is_token_present():
            token = MTTokenHandler().get_token()
            data = token['data']
            username_f = data['user_firstname']
            email = data['user_email']
            exp = data['exp']


            config = mw.addonManager.getConfig(__name__)

            trial_status = config.get("user").get("trial-status")

            access = token['data']['access']

            addon_version = MTCONFIG.get("ADDON_VERSION")


            access_texts = self.get_access_texts(access, trial_status)

            print_access_txt = access_texts[0]
            print_access_class = access_texts[1]

            
            exp_text = ""
            
            exp_text = self.get_exp_Text(exp)

            #needs to be uptodate

            count = config.get("user").get('view-count')
            settings_autounfold = MTCONFIG.get("SETTINGS").get("cards_mtbutton_autounfold")
            if settings_autounfold is True:
                settings_autounfold = "checked"
            else:
                settings_autounfold = ""


            settings_autopin = MTCONFIG.get("SETTINGS").get("cards_mtbutton_autoankiboxpin")
            if settings_autopin is True:
                settings_autopin = "checked"
            else:
                settings_autopin = ""



            settings_dev_emulate_iframe = MTCONFIG.get("SETTINGS").get("cards_dev_emulate_iframe")
            if settings_dev_emulate_iframe is True:
                settings_dev_emulate_iframe = "checked"
            else:
                settings_dev_emulate_iframe = ""


            settings_dev_emulate_iframe_gdpr = MTCONFIG.get("SETTINGS").get("cards_dev_emulate_iframe_gdpr")
            if settings_dev_emulate_iframe_gdpr is True:
                settings_dev_emulate_iframe_gdpr = "checked"
            else:
                settings_dev_emulate_iframe_gdpr = ""
                
        
        try:
            import uuid
            machine_id = uuid.getnode()
            # Inject the data into the HTML
            html = html.format(addon_version=addon_version, username_f=username_f, email=email, exp_text=exp_text, count=count, settings_mtbutton_autounfold = settings_autounfold, settings_mtbutton_autoankiboxpin = settings_autopin, settings_dev_emulate_iframe = settings_dev_emulate_iframe,settings_dev_emulate_iframe_gdpr = settings_dev_emulate_iframe_gdpr,url_signup=MTCONFIG.get("MT_URL_SIGNUP"), url_help_decks=MTCONFIG.get("MT_URL_HELP_DECKS") ,url_help_howto=MTCONFIG.get("MT_URL_HELP_HOWTO"), access_text = print_access_txt, access_class=print_access_class)

            #print("setting html")
            self.browser.setHtml(html, base_url)
            self.currentWindowType = MAINWINDOWTYPES.get('profile')
            
            #print("set html")

            self.raise_()
            self.activateWindow()
        
        except Exception as e:
            print("Error in loading MTWindow:", e)
    
    def get_exp_Text(self, exp):
            exp_text = ""
            timestamp = exp  

            dt_object = datetime.fromtimestamp(timestamp)
            # Get the current date (without time)
            current_date = datetime.now().date()

            readable_date = dt_object.strftime('%d.%m.%Y %H:%M')

            if dt_object.date() < current_date:
                exp_text = f"Dein Login in Anki war bis <b>{readable_date}</b> aktiv. Jetzt mit 1-Klick erneuern."
            else:
                exp_text = f"Dein Login in Anki ist bis <b>{readable_date}</b> aktiv."
            return exp_text
    
    def get_access_texts(self, access, trial_status):
            #print("access, inside gett_access_texts")
            #print(access)
            if access is None or not access:
                if (trial_status == MTCONFIG['MT_ANKIBOX_TRIAL_FINISHED']):
                    print_access_txt = "<span class='text-minor'>Du hast kein aktives Lernpaket und Deine Testphase bereits abgeschlossen.</span><br><span class='text-major'>Starte mit einem Lernpaket, um Meditricks in Deine Anki-Karten zu integrieren.</span>"
                    print_access_class = "no-access trial-finished"
                
                elif (trial_status == MTCONFIG['MT_ANKIBOX_TRIAL_ONGOING']):
                    print_access_txt = "Du hast kein aktives Lernpaket."
                    print_access_class = "no-access trial-started"
                else:
                    print_access_txt = "Du hast kein aktives Lernpaket."
                    print_access_class = "no-access start-trial"
            else:
                print_access_txt = mw.MTAppContext.getAccessText(access)
                print_access_class = "has-access"

            return [print_access_txt, print_access_class]
    
    def update_relogin_details(self):
        if self.currentWindowType != MAINWINDOWTYPES.get('profile'):
            return

        if MTTokenHandler().is_token_present():
            token = MTTokenHandler().get_token()
            data = token['data']
            #username_f = data['user_firstname']
            #email = data['user_email']
            exp = data['exp']

            config = mw.addonManager.getConfig(__name__)

            trial_status = config.get("user").get("trial-status")

            access = token['data']['access']

            access_texts = self.get_access_texts(access, trial_status)

            print_access_txt = access_texts[0]
            print_access_class = access_texts[1]

                
            exp_text = ""
                
            exp_text = self.get_exp_Text(exp)

            count = config.get("user").get('view-count')

            js_code = f"""
                update_relogin_details('{print_access_txt}', '{print_access_class}', '{exp_text}', '{count}');
            """
            
            self.browser.page().runJavaScript(js_code)

    def profile_updateViewCounter(self):
        if self.currentWindowType != MAINWINDOWTYPES.get('profile'):
            return

        config = mw.addonManager.getConfig(__name__)


        count = config.get("user").get('view-count')

        js_code = f"""
                update_viewcounter('{count}');
            """
            
        self.browser.page().runJavaScript(js_code)

    def update_startTrialSuccess(self):
        #check for windowtype profile
        #print("in mtmainwindow now")
        if self.currentWindowType != MAINWINDOWTYPES.get('profile'):
            return
        
        js_code = f"""
            animate_startTrialSuccess();
        """
        
        self.browser.page().runJavaScript(js_code)

    def login_display_result_success(self, name):
        #check for windowtype login
        #print("in mtmainwindow now")
        if self.currentWindowType != MAINWINDOWTYPES.get('login'):
            return
        
        

        
        js_code = f"""
            stopLoading();
            displaySuccess('{name}');
        """
        
        self.browser.page().runJavaScript(js_code)

    def login_display_result_fail(self, errorcode, message):
        #check for windowtype login
        #print("in mtmainwindow now")
        if self.currentWindowType != MAINWINDOWTYPES.get('login'):
            return
        js_code = ""
        if errorcode == "bad_username":
            js_code = """
            stopLoading();
            displayMsg('Falsche Zugangsdaten');
        """
        else:
            js_code = f"""
            stopLoading();
            displayMsg('{message}');
        """
        
        self.browser.page().runJavaScript(js_code)
    
    def profile_display_relogin_success(self, message = None):
        #check for windowtype profile
        #print("in mtmainwindow now")
        if self.currentWindowType != MAINWINDOWTYPES.get('profile'):
            return
        
        js_code = f"""
            animate_stopRelogin();
            displaySuccess_relogin();
        """
        
        self.browser.page().runJavaScript(js_code)

        self.update_relogin_details()

    def profile_display_relogin_fail(self, message = None):
        #check for windowtype profile
        #print("in mtmainwindow now")
        if self.currentWindowType != MAINWINDOWTYPES.get('profile'):
            return
        js_code = ""
        js_code = f"""
            animate_stopRelogin();
            displayMsg_relogin('{message}');
        """
        
        self.browser.page().runJavaScript(js_code)